@@ -30,20 +30,13 @@
 #include "main.h"
 #include "extern.h"
 
-/* Goal-column to arrive when `prev/next-line' functions are used.  */
-static int cur_goalc;
-
 DEFUN ("beginning-of-line", beginning_of_line)
 /*+
 Move point to beginning of current line.
 +*/
 {
   set_buffer_pt (cur_bp, line_beginning_position (uniarg));
-
-  /* Change the `goalc' to the beginning of line for next
-     `prev/next-line' calls.  */
-  thisflag |= FLAG_DONE_CPCN;
-  cur_goalc = 0;
+  set_buffer_goalc (cur_bp, 0);
 }
 END_DEFUN
 
@@ -53,11 +46,7 @@ Move point to end of current line.
 +*/
 {
   set_buffer_pt (cur_bp, line_end_position (uniarg));
-
-  /* Change the `goalc' to the end of line for next
-     `prev/next-line' calls.  */
-  thisflag |= FLAG_DONE_CPCN;
-  cur_goalc = INT_MAX;
+  set_buffer_goalc (cur_bp, SIZE_MAX);
 }
 END_DEFUN
 
@@ -69,8 +58,9 @@ get_goalc_bp (Buffer * bp, Point pt)
 {
   size_t col = 0, t = tab_width (bp), i;
   const char *sp = astr_cstr (get_line_text (pt.p));
+  size_t end = MIN (pt.o, astr_len (get_line_text (pt.p)));
 
-  for (i = 0; i < pt.o; i++)
+  for (i = 0; i < end; i++)
     {
       if (sp[i] == '\t')
         col |= t - 1;
@@ -91,20 +81,20 @@ get_goalc (void)
  * tabulations.
  */
 static void
-goto_goalc (size_t goalc)
+goto_goalc (void)
 {
   Point pt = get_buffer_pt (cur_bp);
   size_t i, col = 0, t = tab_width (cur_bp);
 
   for (i = 0; i < astr_len (get_line_text (pt.p)); i++)
     {
-      if (col == goalc)
+      if (col == get_buffer_goalc (cur_bp))
         break;
       else if (astr_get (get_line_text (pt.p), i) == '\t')
         {
           size_t w;
           for (w = t - col % t; w > 0; w--)
-            if (++col == goalc)
+            if (++col == get_buffer_goalc (cur_bp))
               break;
         }
       else
@@ -115,29 +105,62 @@ goto_goalc (size_t goalc)
   set_buffer_pt (cur_bp, pt);
 }
 
-int
-previous_line (void)
+static bool
+move_line (int n)
 {
-  if (get_line_prev (get_buffer_pt (cur_bp).p) != get_buffer_lines (cur_bp))
+  bool ok = true;
+  int dir;
+  Point pt = get_buffer_pt (cur_bp);
+
+  if (n == 0)
+    return false;
+  else if (n > 0)
     {
-      Point pt;
-
-      thisflag |= FLAG_DONE_CPCN | FLAG_NEED_RESYNC;
-
-      if (!(lastflag & FLAG_DONE_CPCN))
-        cur_goalc = get_goalc ();
-
-      pt = get_buffer_pt (cur_bp);
-      pt.p = get_line_prev (pt.p);
-      pt.n--;
-      set_buffer_pt (cur_bp, pt);
-
-      goto_goalc (cur_goalc);
-
-      return true;
+      dir = 1;
+      if ((size_t) n > get_buffer_last_line (cur_bp) - pt.n)
+        {
+          ok = false;
+          n = get_buffer_last_line (cur_bp) - pt.n;
+        }
+    }
+  else
+    {
+      dir = -1;
+      n = -n;
+      if ((size_t) n > pt.n)
+        {
+          ok = false;
+          n = pt.n;
+        }
     }
 
-  return false;
+  for (; n > 0; n--)
+    {
+      Point pt = get_buffer_pt (cur_bp);
+      pt.p = (dir > 0 ? get_line_next : get_line_prev) (pt.p);
+      pt.n += dir;
+      set_buffer_pt (cur_bp, pt);
+    }
+
+  if (last_command () != F_next_line && last_command () != F_previous_line)
+    set_buffer_goalc (cur_bp, get_goalc ());
+  goto_goalc ();
+
+  thisflag |= FLAG_NEED_RESYNC;
+
+  return ok;
+}
+
+bool
+previous_line (void)
+{
+  return move_line (-1);
+}
+
+bool
+next_line (void)
+{
+  return move_line (1);
 }
 
 DEFUN ("previous-line", previous_line)
@@ -148,51 +171,13 @@ the cursor is positioned after the character in that line which spans this
 column, or at the end of the line if it is not long enough.
 +*/
 {
-  if (uniarg < 0)
-    return FUNCALL_ARG (next_line, -uniarg);
-
-  if (!bobp ())
-    {
-      int i;
-
-      for (i = 0; i < uniarg; i++)
-        if (!previous_line ())
-          {
-            thisflag |= FLAG_DONE_CPCN;
-            FUNCALL (beginning_of_line);
-            break;
-          }
-    }
-  else if (lastflag & FLAG_DONE_CPCN)
-    thisflag |= FLAG_DONE_CPCN;
+  if (uniarg < 0 || !bobp ())
+    ok = execute_with_uniarg (false, uniarg, previous_line, next_line);
+  if (ok == leNIL)
+    FUNCALL (beginning_of_line);
 }
 END_DEFUN
 
-int
-next_line (void)
-{
-  if (get_line_next (get_buffer_pt (cur_bp).p) != get_buffer_lines (cur_bp))
-    {
-      Point pt;
-
-      thisflag |= FLAG_DONE_CPCN | FLAG_NEED_RESYNC;
-
-      if (!(lastflag & FLAG_DONE_CPCN))
-        cur_goalc = get_goalc ();
-
-      pt = get_buffer_pt (cur_bp);
-      pt.p = get_line_next (pt.p);
-      pt.n++;
-      set_buffer_pt (cur_bp, pt);
-
-      goto_goalc (cur_goalc);
-
-      return true;
-    }
-
-  return false;
-}
-
 DEFUN ("next-line", next_line)
 /*+
 Move cursor vertically down one line.
@@ -201,25 +186,10 @@ the cursor is positioned after the character in that line which spans this
 column, or at the end of the line if it is not long enough.
 +*/
 {
-  if (uniarg < 0)
-    return FUNCALL_ARG (previous_line, -uniarg);
-
-  if (!eobp ())
-    {
-      int i;
-
-      for (i = 0; i < uniarg; i++)
-        if (!next_line ())
-          {
-            int old = cur_goalc;
-            thisflag |= FLAG_DONE_CPCN;
-            FUNCALL (end_of_line);
-            cur_goalc = old;
-            break;
-          }
-    }
-  else if (lastflag & FLAG_DONE_CPCN)
-    thisflag |= FLAG_DONE_CPCN;
+  if (uniarg < 0 || !eobp ())
+    ok = execute_with_uniarg (false, uniarg, next_line, previous_line);
+  if (ok == leNIL)
+    FUNCALL (end_of_line);
 }
 END_DEFUN
 
@@ -260,13 +230,13 @@ Position 1 is the beginning of the buffer.
 END_DEFUN
 
 DEFUN_ARGS ("goto-line", goto_line,
-            INT_ARG (n))
+            INT_OR_UNIARG (n))
 /*+
 Goto line arg, counting from line 1 at beginning of buffer.
 +*/
 {
-  INT_INIT (n)
-  else
+  INT_OR_UNIARG_INIT (n);
+  if (noarg)
     {
       n = minibuf_read_number ("Goto line: ");
       if (n == LONG_MAX - 1)
@@ -277,19 +247,9 @@ Goto line arg, counting from line 1 at beginning of buffer.
     ok = leNIL;
   else
     {
-      Point pt;
-
-      if (n <= 0)
-        n = 1;
-      n--; /* Re-base to counting from zero */
-      pt = get_buffer_pt (cur_bp);
-      if (pt.n > (size_t) n)
-        ngotoup (pt.n - (size_t) n);
-      else if (pt.n < (size_t) n)
-        ngotodown ((size_t) n - pt.n);
-      pt = get_buffer_pt (cur_bp);
-      pt.o = 0;
-      set_buffer_pt (cur_bp, pt);
+      Point pt = get_buffer_pt (cur_bp);
+      move_line ((size_t) (MAX (n, 1) - 1) - pt.n);
+      FUNCALL (beginning_of_line);
     }
 }
 END_DEFUN
@@ -301,7 +261,7 @@ void
 gotobob (void)
 {
   set_buffer_pt (cur_bp, point_min ());
-  thisflag |= FLAG_DONE_CPCN | FLAG_NEED_RESYNC;
+  thisflag |= FLAG_NEED_RESYNC;
 }
 
 DEFUN ("beginning-of-buffer", beginning_of_buffer)
@@ -321,7 +281,7 @@ void
 gotoeob (void)
 {
   set_buffer_pt (cur_bp, point_max ());
-  thisflag |= FLAG_DONE_CPCN | FLAG_NEED_RESYNC;
+  thisflag |= FLAG_NEED_RESYNC;
 }
 
 DEFUN ("end-of-buffer", end_of_buffer)
@@ -334,52 +294,43 @@ Move point to the end of the buffer; leave mark at previous position.
 }
 END_DEFUN
 
-int
-backward_char (void)
+static bool
+move_char (int dir)
 {
-  if (!bolp ())
+  if (dir > 0 ? !eolp () : !bolp ())
     {
       Point pt = get_buffer_pt (cur_bp);
-      pt.o--;
+      pt.o += dir;
       set_buffer_pt (cur_bp, pt);
       return true;
     }
-  else if (!bobp ())
+  else if (dir > 0 ? !eobp () : !bobp ())
     {
       Point pt = get_buffer_pt (cur_bp);
       thisflag |= FLAG_NEED_RESYNC;
-      pt.p = get_line_prev (pt.p);
-      pt.n--;
+      pt.p = (dir > 0 ? get_line_next : get_line_prev) (pt.p);
+      pt.n += dir;
       set_buffer_pt (cur_bp, pt);
-      FUNCALL (end_of_line);
+      if (dir > 0)
+        FUNCALL (beginning_of_line);
+      else
+        FUNCALL (end_of_line);
       return true;
     }
 
   return false;
 }
 
-int
+bool
+backward_char (void)
+{
+  return move_char (-1);
+}
+
+bool
 forward_char (void)
 {
-  if (!eolp ())
-    {
-      Point pt = get_buffer_pt (cur_bp);
-      pt.o++;
-      set_buffer_pt (cur_bp, pt);
-      return true;
-    }
-  else if (!eobp ())
-    {
-      Point pt = get_buffer_pt (cur_bp);
-      thisflag |= FLAG_NEED_RESYNC;
-      pt.p = get_line_next (pt.p);
-      pt.n++;
-      set_buffer_pt (cur_bp, pt);
-      FUNCALL (beginning_of_line);
-      return true;
-    }
-
-  return false;
+  return move_char (1);
 }
 
 DEFUN ("backward-char", backward_char)
@@ -406,45 +357,22 @@ On reaching end of buffer, stop and signal error.
 }
 END_DEFUN
 
-int
-ngotoup (size_t n)
-{
-  for (; n > 0; n--)
-    if (get_line_prev (get_buffer_pt (cur_bp).p) != get_buffer_lines (cur_bp))
-      FUNCALL (previous_line);
-    else
-      return false;
-
-  return true;
-}
-
-int
-ngotodown (size_t n)
-{
-  for (; n > 0; n--)
-    if (get_line_next (get_buffer_pt (cur_bp).p) != get_buffer_lines (cur_bp))
-      FUNCALL (next_line);
-    else
-      return false;
-
-  return true;
-}
-
-static int
+static bool
 scroll_down (void)
 {
-  if (get_buffer_pt (cur_bp).n > 0)
-    return ngotoup (get_window_eheight (cur_wp)) ? true : false;
+  if (get_buffer_pt (cur_bp).n > get_window_eheight (cur_wp))
+    return move_line (-get_window_eheight (cur_wp));
 
   minibuf_error ("Beginning of buffer");
   return false;
 }
 
-static int
+static bool
 scroll_up (void)
 {
-  if (get_buffer_pt (cur_bp).n < get_buffer_last_line (cur_bp))
-    return ngotodown (get_window_eheight (cur_wp)) ? true : false;
+  if (get_buffer_pt (cur_bp).n <
+      get_buffer_last_line (cur_bp) - get_window_eheight (cur_wp))
+    return move_line (get_window_eheight (cur_wp));
 
   minibuf_error ("End of buffer");
   return false;
