@@ -125,69 +125,83 @@ typedef le * (*Function) (long uniarg, le * list);
 
 /* Define an interactive function. */
 #define DEFUN(zile_func, c_func) \
-        DEFUN_ARGS(zile_func, c_func, )
+  DEFUN_ARGS(zile_func, c_func, )
 #define DEFUN_ARGS(zile_func, c_func, args) \
-        le * F_ ## c_func (long uniarg GCC_UNUSED, le *arglist GCC_UNUSED) \
-        { \
-          le * ok = leT; \
-          args
-#define END_DEFUN \
-          return ok; \
-        }
+  le * F_ ## c_func (long uniarg GCC_UNUSED, le *arglist GCC_UNUSED)    \
+  {                                                                     \
+    le * ok = leT;                                                      \
+    args
+#define END_DEFUN    \
+    return ok;       \
+  }
 
 /* Define a non-user-visible function. */
 #define DEFUN_NONINTERACTIVE(zile_func, c_func) \
-        DEFUN(zile_func, c_func)
+  DEFUN(zile_func, c_func)
 #define DEFUN_NONINTERACTIVE_ARGS(zile_func, c_func, args) \
-        DEFUN_ARGS(zile_func, c_func, args)
+  DEFUN_ARGS(zile_func, c_func, args)
 
 /* String argument. */
-#define STR_ARG(name) \
-        const char *name = NULL; \
-        bool free_ ## name = true;
-#define STR_INIT(name) \
-        if (arglist && arglist->next) \
-          { \
-            name = arglist->next->data; \
-            arglist = arglist->next; \
-            free_ ## name = false; \
-          }
-#define STR_FREE(name) \
-        if (free_ ## name) \
-          free ((char *) name);
+#define STR_ARG(name)                   \
+  const char *name = NULL;              \
+  bool free_ ## name = true;
+#define STR_INIT(name)                  \
+  if (arglist && arglist->next)         \
+    {                                   \
+      name = arglist->next->data;       \
+      arglist = arglist->next;          \
+      free_ ## name = false;            \
+    }
+#define STR_FREE(name)                  \
+  if (free_ ## name)                    \
+    free ((char *) name);
 
 /* Integer argument. */
 #define INT_ARG(name) \
-        long name = 1;
+  long name = 1;
 #define INT_INIT(name) \
-        if (arglist && arglist->next) \
-          { \
-            const char *s = arglist->next->data; \
-            arglist = arglist->next; \
-            name = strtol (s, NULL, 10); \
-            if (name == LONG_MAX) \
-              ok = leNIL; \
-          }
+  if (arglist && arglist->next)                 \
+    {                                           \
+      const char *s = arglist->next->data;      \
+      arglist = arglist->next;                  \
+      if (s != NULL)                            \
+        name = strtol (s, NULL, 10);            \
+      if (name == LONG_MAX)                     \
+        ok = leNIL;                             \
+    }
+
+/* Integer argument which can either be argument or uniarg. */
+#define INT_OR_UNIARG(name) \
+  long name = 1;            \
+  bool noarg = false;
+#define INT_OR_UNIARG_INIT(name)                             \
+  INT_INIT (name)                                            \
+  else                                                       \
+    {                                                        \
+      if (!(lastflag & FLAG_SET_UNIARG) && arglist == NULL)  \
+        noarg = true;                                        \
+      name = uniarg;                                         \
+    }
 
 /* Boolean argument. */
-#define BOOL_ARG(name) \
-        bool name = true;
-#define BOOL_INIT(name) \
-        if (arglist && arglist->next) \
-          { \
-            const char *s = arglist->next->data; \
-            arglist = arglist->next; \
-            if (strcmp (s, "nil") == 0) \
-              name = false; \
-          }
+#define BOOL_ARG(name)                          \
+  bool name = true;
+#define BOOL_INIT(name)                         \
+  if (arglist && arglist->next)                 \
+    {                                           \
+      const char *s = arglist->next->data;      \
+      arglist = arglist->next;                  \
+      if (s != NULL && strcmp (s, "nil") == 0)  \
+        name = false;                           \
+    }
 
 /* Call an interactive function. */
 #define FUNCALL(c_func)                         \
-        F_ ## c_func (1, NULL)
+  F_ ## c_func (1, leNIL)
 
 /* Call an interactive function with a universal argument. */
 #define FUNCALL_ARG(c_func, uniarg)             \
-        F_ ## c_func (uniarg, NULL)
+  F_ ## c_func (uniarg, leNIL)
 
 /*--------------------------------------------------------------------------
  * Keyboard handling.
@@ -236,22 +250,19 @@ typedef le * (*Function) (long uniarg, le * list);
  *--------------------------------------------------------------------------*/
 
 /* Global flags, stored in thisflag and lastflag. */
-#define FLAG_DONE_CPCN		0001	/* Last command was C-p or C-n. */
-#define FLAG_DONE_KILL		0002	/* The last command was a kill. */
-#define FLAG_NEED_RESYNC	0004	/* A resync is required. */
-#define FLAG_QUIT		0010	/* The user has asked to quit. */
-#define FLAG_SET_UNIARG		0020	/* The last command modified the
+#define FLAG_NEED_RESYNC	0001	/* A resync is required. */
+#define FLAG_QUIT		0002	/* The user has asked to quit. */
+#define FLAG_SET_UNIARG		0004	/* The last command modified the
                                            universal arg variable `uniarg'. */
-#define FLAG_UNIARG_EMPTY	0040	/* Current universal arg is just C-u's
+#define FLAG_UNIARG_EMPTY	0010	/* Current universal arg is just C-u's
                                            with no number. */
-#define FLAG_DEFINING_MACRO	0100	/* We are defining a macro. */
+#define FLAG_DEFINING_MACRO	0020	/* We are defining a macro. */
 
 /*
  * Zile font codes
- * Designed to fit in an int, leaving room for a char underneath.
  */
-#define FONT_NORMAL		0x000
-#define FONT_REVERSE		0x100
+#define FONT_NORMAL		0000
+#define FONT_REVERSE		0001
 
 /* Default waitkey pause in ds */
 #define WAITKEY_DEFAULT 20
