@@ -29,10 +29,19 @@
 #else
 #include <curses.h>
 #endif
+#include "gl_array_list.h"
 
 #include "main.h"
 #include "extern.h"
 
+static gl_list_t key_buf;
+
+size_t
+term_buf_len (void)
+{
+  return gl_list_size (key_buf);
+}
+
 void
 term_move (size_t y, size_t x)
 {
@@ -66,14 +75,7 @@ term_addch (int c)
 void
 term_attrset (size_t attr)
 {
-  unsigned long a = 0;
-
-  if (attr == FONT_NORMAL)
-    a = 0;
-  else if (attr ==  FONT_REVERSE)
-    a = A_REVERSE;
-
-  attrset (a);
+  attrset (attr == FONT_REVERSE ? A_REVERSE : 0);
 }
 
 void
@@ -93,6 +95,7 @@ term_init (void)
   meta (stdscr, true);
   intrflush (stdscr, false);
   keypad (stdscr, true);
+  key_buf = gl_list_create_empty (GL_ARRAY_LIST, NULL, NULL, NULL, true);
 }
 
 void
@@ -104,6 +107,7 @@ term_close (void)
   term_refresh ();
 
   /* Free memory and finish with ncurses. */
+  gl_list_free (key_buf);
   endwin ();
 }
 
@@ -337,6 +341,23 @@ keytocodes (size_t key, int ** codevec)
   return p - *codevec;
 }
 
+static int
+get_char (void)
+{
+  int c;
+  size_t size = term_buf_len ();
+
+  if (size > 0)
+    {
+      c = (int) gl_list_get_at (key_buf, size - 1);
+      gl_list_remove_at (key_buf, size - 1);
+    }
+  else
+    c = getch ();
+
+  return c;
+}
+
 size_t
 term_xgetkey (int mode, size_t timeout)
 {
@@ -348,7 +369,8 @@ term_xgetkey (int mode, size_t timeout)
 
       if (mode & GETKEY_DELAYED)
         wtimeout (stdscr, (int) timeout * 100);
-      c = getch ();
+
+      c = get_char ();
       if (mode & GETKEY_DELAYED)
         wtimeout (stdscr, -1);
 
@@ -367,7 +389,7 @@ term_xgetkey (int mode, size_t timeout)
         {
           key = codetokey (c);
           while (key == KBD_META)
-            key = codetokey (getch ()) | KBD_META;
+            key = codetokey (get_char ()) | KBD_META;
         }
       break;
     }
@@ -382,7 +404,7 @@ term_ungetkey (size_t key)
   size_t i, n = keytocodes (key, &codes);
 
   for (i = n; i > 0; i--)
-    ungetch (codes[i - 1]);
+    gl_list_add_last (key_buf, (void *) codes[i - 1]);
 
   free (codes);
 }
